@@ -67,11 +67,12 @@ void phaseWithRef(EagleParams &params, Timer &timer, double t0, int argc, char *
   string outFile = params.outPrefix + "." + params.vcfOutSuffix;
   vector < vector < pair <int, int> > > conPSall; double snpRate;
   vector <bool> isTmpPhased;
-  SyncedVcfData vcfData(params.vcfRef, params.vcfTarget, params.allowRefAltSwap, params.chrom,
-			params.chromX, params.bpStart-params.bpFlanking,
+  SyncedVcfData vcfData(params.vcfRef, params.vcfTarget, params.vcfExclude, params.allowRefAltSwap,
+			params.chrom, params.chromX, params.bpStart-params.bpFlanking,
 			params.bpEnd+params.bpFlanking, params.geneticMapFile,
 			params.cMmax==0 ? 1 : params.cMmax, tmpFile, params.vcfWriteMode,
 			params.usePS, conPSall, snpRate, params.outputUnphased, isTmpPhased);
+  cout << endl << "Time for reading input: " << timer.update_time() << " sec" << endl << endl;
 
   Eagle eagle(vcfData.getNref(), vcfData.getNtarget(), vcfData.getMseg64(),
 	      vcfData.getGenoBits(), vcfData.getSeg64cMvecs(), params.pErr);
@@ -187,7 +188,7 @@ void phaseWithRef(EagleParams &params, Timer &timer, double t0, int argc, char *
   timer.update_time();
   cout << "Writing " << params.vcfOutSuffix << " output to " << outFile << endl;
   eagle.writeVcf(tmpFile, isTmpPhased, outFile, params.chromX, params.bpStart, params.bpEnd,
-		 params.vcfWriteMode, params.noImpMissing, argc, argv);
+		 params.vcfWriteMode, params.noImpMissing, params.keepMissingPloidyX, argc, argv);
   cout << "Time for writing output: " << timer.update_time() << endl;
 
   cout << "Total elapsed time for analysis = " << (timer.get_time()-t0) << " sec" << endl;
@@ -215,7 +216,7 @@ int main(int argc, char *argv[]) {
   cout << "                      +-----------------------------+" << endl;
   cout << endl;
 
-  cout << "Copyright (C) 2015-2016 Harvard University." << endl;
+  cout << "Copyright (C) 2015-2017 Harvard University." << endl;
   cout << "Distributed under the GNU GPLv3+ open source license." << endl << endl;
 
   //cout << "Boost version: " << BOOST_LIB_VERSION << endl;
@@ -285,6 +286,9 @@ int main(int argc, char *argv[]) {
     int bpSpan = genoData.getSnps().back().physpos - genoData.getSnps()[0].physpos;
     double snpsPerMb = genoData.getSnps().size() / (bpSpan*1e-6);
     if (snpsPerMb > 1000) params.pbwtOnly = true;
+
+    if (genoData.getN() > 200000U) params.pbwtOnly = true;
+
     if (params.pbwtOnly) params.runStep2 = 0;
 
     // if --runStep2 hasn't yet been set, SNP density must be low; run Step 2 unless too few chunks
@@ -499,7 +503,8 @@ int main(int argc, char *argv[]) {
       string outFile = params.outPrefix + "." + params.vcfOutSuffix;
       cout << "Writing " << params.vcfOutSuffix << " output to " << outFile << endl;
       eagle.writeVcfNonRef(params.vcfFile, outFile, params.chrom, params.chromX, params.bpStart,
-			   params.bpEnd, params.vcfWriteMode, argc, argv);
+			   params.bpEnd, params.vcfWriteMode, params.keepMissingPloidyX, argc,
+			   argv);
     }
     else {
       cout << "Writing .haps.gz and .sample output" << endl; timer.update_time();
