@@ -159,17 +159,19 @@ namespace EAGLE {
       }
   }
 
+  // fills in chrom if chrom==0
   vector < pair <int, int> > SyncedVcfData::processVcfs
-  (const string &vcfRef, const string &vcfTarget, bool allowRefAltSwap, int chrom, int chromX,
-   double bpStart, double bpEnd, vector <bool> &hapsRef, vector <uchar> &genosTarget,
-   const string &tmpFile, const string &writeMode, int usePS,
+  (const string &vcfRef, const string &vcfTarget, const string &vcfExclude, bool allowRefAltSwap,
+   int &chrom, int chromX, double bpStart, double bpEnd, vector <bool> &hapsRef,
+   vector <uchar> &genosTarget, const string &tmpFile, const string &writeMode, int usePS,
    vector < vector < pair <int, int> > > &conPSall, bool outputUnphased,
    vector <bool> &isTmpPhased) {
 
     vector < pair <int, int> > chrBps;
 
     bcf_srs_t *sr = bcf_sr_init();
-    bcf_sr_set_opt(sr, BCF_SR_PAIR_LOGIC, BCF_SR_PAIR_BOTH_REF);
+    bcf_sr_set_opt(sr, BCF_SR_PAIR_LOGIC,
+		   allowRefAltSwap ? BCF_SR_PAIR_BOTH_REF : BCF_SR_PAIR_EXACT);
     bcf_sr_set_opt(sr, BCF_SR_REQUIRE_IDX);
 
     if ( chrom!=0 )
@@ -181,7 +183,7 @@ namespace EAGLE {
             ksprintf(&str,"X:%d-%d,chrX:%d-%d",(uint32_t)bpStart,(uint32_t)bpEnd,(uint32_t)bpStart,(uint32_t)bpEnd);
         }
         else
-            ksprintf(&str,"%d:%d-%d",chrom,(uint32_t)bpStart,(uint32_t)bpEnd);
+            ksprintf(&str,"%d:%d-%d,chr%d:%d-%d",chrom,(uint32_t)bpStart,(uint32_t)bpEnd,chrom,(uint32_t)bpStart,(uint32_t)bpEnd);
         if ( bcf_sr_set_regions(sr, str.s, 0)!=0 )
         {
             cerr << "ERROR: failed to initialize the region:" << str.s;
@@ -200,6 +202,12 @@ namespace EAGLE {
 	   << bcf_sr_strerror(sr->errnum) << endl;
       exit(1);
     }
+    bool useExclude = !vcfExclude.empty();
+    if (useExclude && !bcf_sr_add_reader(sr, vcfExclude.c_str())) {
+      cerr << "ERROR: Could not open " << vcfExclude << " for reading: "
+	   << bcf_sr_strerror(sr->errnum) << endl;
+      exit(1);
+    }
 
     bcf_hdr_t *ref_hdr = bcf_sr_get_header(sr, 0);
     bcf_hdr_t *tgt_hdr = bcf_sr_get_header(sr, 1);
@@ -229,7 +237,7 @@ namespace EAGLE {
     cout << "Target samples: Ntarget = " << Ntarget << endl;
 
     M = 0;
-    uint MtargetOnly = 0, MrefOnly = 0, MmultiAllelicTgt = 0,
+    uint Mexclude = 0, MtargetOnly = 0, MrefOnly = 0, MmultiAllelicTgt = 0,
       MmultiAllelicRef = 0, MmonomorphicRef = 0;
     uint MwithMissingRef = 0, MwithUnphasedRef = 0, MnotInRegion = 0, MnotOnChrom = 0;
     uint MrefAltError = 0, numRefAltSwaps = 0;
@@ -244,6 +252,11 @@ namespace EAGLE {
       {
 	bcf1_t *ref = bcf_sr_get_line(sr, 0);
 	bcf1_t *tgt = bcf_sr_get_line(sr, 1);
+	if (useExclude && bcf_sr_get_line(sr, 2)) {
+	  Mexclude++;
+	  if (outputUnphased && tgt) { bcf_write(out, tgt_hdr, tgt); isTmpPhased.push_back(false); }
+	  continue;
+	}
 	if ( !ref ) {
 	  //fprintf(stderr, "onlyT .. %s:%d\n", bcf_seqname(tgt_hdr, tgt), tgt->pos+1);
 	  bcf_unpack(tgt, BCF_UN_STR); // unpack thru ALT
@@ -261,53 +274,47 @@ namespace EAGLE {
 	}
 	//fprintf(stderr, "match .. %s:%d\n", bcf_seqname(ref_hdr, ref), ref->pos+1);
 
-	// filter out multi-allelic and monomorphic markers
+	// deal with multi-allelic and monomorphic markers
+	// drop multi-allelic target markers
 	int ntgt_gt = bcf_get_genotypes(tgt_hdr, tgt, &tgt_gt, &mtgt_gt);
 	if (tgt->n_allele > 2) {
 	  MmultiAllelicTgt++;
 	  if (outputUnphased) { bcf_write(out, tgt_hdr, tgt); isTmpPhased.push_back(false); }
 	  continue;
 	}
-    if (ref->n_allele==1)
-    {
         // drop monomorphic reference markers
-        MmonomorphicRef++;
-	if (outputUnphased) { bcf_write(out, tgt_hdr, tgt); isTmpPhased.push_back(false); }
-        continue;
-    }
-
-    // preserve monomorphic markers if biallelic in the reference panel
-    if (tgt->n_allele < 2)
-    {
-        bcf_update_alleles(tgt_hdr, tgt, (const char**)ref->d.allele,ref->n_allele);
-    }
+	if (ref->n_allele==1) {
+	  MmonomorphicRef++;
+	  if (outputUnphased) { bcf_write(out, tgt_hdr, tgt); isTmpPhased.push_back(false); }
+	  continue;
+	}
+	// preserve monomorphic markers if biallelic in the reference panel
+	if (tgt->n_allele < 2) {
+	  bcf_update_alleles(tgt_hdr, tgt, (const char**)ref->d.allele,ref->n_allele);
+	}
+	// drop multi-allelic reference markers
+	if (ref->n_allele > 2) {
+	  MmultiAllelicRef++;
+	  if (outputUnphased) { bcf_write(out, tgt_hdr, tgt); isTmpPhased.push_back(false); }
+	  continue;
+	}
 
+	// check for REF/ALT swaps
 	bool refAltSwap = false;
-
-        if (allowRefAltSwap) { // perform further error-checking
-	  if (ref->n_allele > 2) {
-	    MmultiAllelicRef++;
-	    if (outputUnphased) { bcf_write(out, tgt_hdr, tgt); isTmpPhased.push_back(false); }
-	    continue;
-	  }
-	  /*
-	  printf("tgt REF=%s, ALT=%s   ref REF=%s, ALT=%s\n", tgt->d.allele[0], tgt->d.allele[1],
-		 ref->d.allele[0], ref->d.allele[1]);
-	  */
-	  if (strcmp(tgt->d.allele[0], ref->d.allele[0]) == 0 &&
-	      strcmp(tgt->d.allele[1], ref->d.allele[1]) == 0) {
-	    refAltSwap = false;
-	  }
-	  else if (strcmp(tgt->d.allele[0], ref->d.allele[1]) == 0 &&
-		   strcmp(tgt->d.allele[1], ref->d.allele[0]) == 0) {
-	    refAltSwap = true;
-	    numRefAltSwaps++;
-	  }
-	  else {
-	    MrefAltError++;
-	    if (outputUnphased) { bcf_write(out, tgt_hdr, tgt); isTmpPhased.push_back(false); }
-	    continue;	    
-	  }
+	if (strcmp(tgt->d.allele[0], ref->d.allele[0]) == 0 &&
+	    strcmp(tgt->d.allele[1], ref->d.allele[1]) == 0) {
+	  refAltSwap = false;
+	}
+	else if (allowRefAltSwap && bcf_is_snp(tgt) && // allow swap if --allowRefAltSwap is set
+		 strcmp(tgt->d.allele[0], ref->d.allele[1]) == 0 &&
+		 strcmp(tgt->d.allele[1], ref->d.allele[0]) == 0) {
+	  refAltSwap = true;
+	  numRefAltSwaps++;
+	}
+	else {
+	  MrefAltError++;
+	  if (outputUnphased) { bcf_write(out, tgt_hdr, tgt); isTmpPhased.push_back(false); }
+	  continue;	    
 	}
 
     // Check the chromosome: if region was requested (chrom is set), synced
@@ -383,7 +390,7 @@ namespace EAGLE {
     cout << endl;
     cout << "SNPs ignored: " << MtargetOnly << " SNPs in target but not reference" << endl;
     if (MtargetOnly > M/10U)
-      cerr << "              --> WARNING: Check REF/ALT agreement between target and ref <--"
+      cerr << "              --> WARNING: Check REF/ALT agreement between target and ref? <--"
 	   << endl;
     cout << "              " << MrefOnly << " SNPs in reference but not target" << endl;
     if (MnotOnChrom)
@@ -397,7 +404,9 @@ namespace EAGLE {
     if (MmonomorphicRef)
       cout << "              " << MmonomorphicRef << " SNPs biallelic in target but monomorphic in ref" << endl;
     if (MrefAltError)
-      cout << "              " << MrefAltError << " SNPs with REF/ALT matching errors" << endl;
+      cout << "              " << MrefAltError << " SNPs with allele mismatches" << endl;
+    if (Mexclude)
+      cout << "              " << Mexclude << " SNPs excluded based on --vcfExclude" << endl;
     cout << endl;
     
     if (MwithMissingRef) {
@@ -492,20 +501,22 @@ namespace EAGLE {
    * reads ref+target vcf data
    * writes target[isec] to tmpFile
    * fills in cM coordinates and seg64cMvecs, genoBits
+   * fills in chrom if chrom==0
    */
-  SyncedVcfData::SyncedVcfData(const string &vcfRef, const string &vcfTarget, bool allowRefAltSwap,
-			       int chrom, int chromX, double bpStart, double bpEnd,
-			       const string &geneticMapFile, double cMmax, const string &tmpFile,
-			       const string &writeMode, int usePS,
-			       vector < vector < pair <int, int> > > &conPSall, double &snpRate,
-			       bool outputUnphased, vector <bool> &isTmpPhased) {
+  SyncedVcfData::SyncedVcfData
+  (const string &vcfRef, const string &vcfTarget, const string &vcfExclude, bool allowRefAltSwap,
+   int &chrom, int chromX, double bpStart, double bpEnd, const string &geneticMapFile,
+   double cMmax, const string &tmpFile, const string &writeMode, int usePS,
+   vector < vector < pair <int, int> > > &conPSall, double &snpRate, bool outputUnphased,
+   vector <bool> &isTmpPhased) {
 
     // perform synced read
     vector <bool> hapsRef;     // M*2*Nref
     vector <uchar> genosTarget; // M*Ntarget
     vector < pair <int, int> > chrBps = 
-      processVcfs(vcfRef, vcfTarget, allowRefAltSwap, chrom, chromX, bpStart, bpEnd, hapsRef,
-		  genosTarget, tmpFile, writeMode, usePS, conPSall, outputUnphased, isTmpPhased);
+      processVcfs(vcfRef, vcfTarget, vcfExclude, allowRefAltSwap, chrom, chromX, bpStart, bpEnd,
+		  hapsRef, genosTarget, tmpFile, writeMode, usePS, conPSall, outputUnphased,
+		  isTmpPhased);
 
     // interpolate genetic coordinates
     vector <double> cMs = processMap(chrBps, geneticMapFile);
